from tfrecord_slow import TfrecordWriter
from tqdm import tqdm
import numpy as np

from flatbuffers import flexbuffers
import Data.Message
import Data.NdArray


def create_message(builder: flexbuffers.Builder, x: np.ndarray):
    builder.Clear()

    # with builder.Map() as map:
    #     map.Key("x")
    #     with map.Map() as map:
    #         map.Key("data")
    #         map.Blob(x.tobytes())

    #         map.Key("shape")
    #         with map.Vector() as vec:
    #             for s in x.shape:
    #                 vec.Int(s)

    #         map.Key("dtype")
    #         map.String(x.dtype.str)
    with builder.Vector() as vec:
        vec.Blob(x.tobytes())
        vec.String(x.dtype.str)



    return builder.Finish()

    # fbs_dtype = builder.CreateString(x.dtype.str)
    # fbs_shape = builder.CreateNumpyVector(np.array(x.shape, dtype=np.int64))
    # fbs_data = builder.CreateByteVector(x.data.tobytes())

    # Data.NdArray.Start(builder)
    # Data.NdArray.AddData(builder, fbs_data)
    # Data.NdArray.AddShape(builder, fbs_shape)
    # Data.NdArray.AddDtype(builder, fbs_dtype)
    # fbs_x = Data.NdArray.End(builder)

    # Data.Message.Start(builder)
    # Data.Message.AddX(builder, fbs_x)
    # fbs_msg = Data.Message.End(builder)

    # builder.Finish(fbs_msg)
    # return builder.Output()


def bench(n: int = 1000):
    x = np.random.rand(1024, 1024)
    # builder = flatbuffers.Builder()
    builder = flexbuffers.Builder()
    with TfrecordWriter.create("/tmp/test_flex_writer.tfrec") as writer:
        for _ in tqdm(range(n)):
            writer.write(create_message(builder, x))


bench()



import numpy as np
from typing import Optional, Sequence


def buffer_to_ndarray(
    buf: memoryview,
    shape: Optional[Sequence[int]] = None,
    dtype: np.dtype = np.uint8,
    copy: bool = True,
) -> np.ndarray:
    x = np.frombuffer(buf, dtype=dtype)
    if copy:
        x = x.copy()
    if shape is not None:
        x = x.reshape(shape)
    return x



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: Data

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

class Message(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = Message()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsMessage(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # Message
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Message
    def X(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from Data.NdArray import NdArray
            obj = NdArray()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

def Start(builder): builder.StartObject(1)
def MessageStart(builder):
    """This method is deprecated. Please switch to Start."""
    return Start(builder)
def AddX(builder, x): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(x), 0)
def MessageAddX(builder, x):
    """This method is deprecated. Please switch to AddX."""
    return AddX(builder, x)
def End(builder): return builder.EndObject()
def MessageEnd(builder):
    """This method is deprecated. Please switch to End."""
    return End(builder)


from tqdm import tqdm
from common import Message
from tfrecord_slow.loaders.raw_loader import RawTfrecordLoader
import Data.Message
import numpy as np


def parse(buf):
    msg = Data.Message.Message.GetRootAs(buf)
    # return {"data": msg.X().Data()}
    x = np.frombuffer(msg.X().DataAsNumpy().data, dtype=msg.X().Dtype().decode())
    x = x.reshape(msg.X().ShapeAsNumpy())
    return x


def bench(n: int = 1000):
    # with TfRecordReader.open("/tmp/test_writer.tfrec") as reader:
    loader = RawTfrecordLoader([open("/tmp/test_fbs_writer.tfrec", "rb")], parse)
    for msg in tqdm(loader, total=n):
        # assert msg.x.to_numpy().shape == (1024, 1024)
        assert msg.shape == (1024, 1024)


bench()



from tqdm import tqdm
from common import Message
from tfrecord_slow.loaders.msgpack_loader import MsgpackTfrecordLoader


def bench(n: int = 1000):
    # with TfRecordReader.open("/tmp/test_writer.tfrec") as reader:
    loader = MsgpackTfrecordLoader(
        [open("/tmp/test_writer.tfrec", "rb")], Message, check_integrity=True
    )
    for msg in tqdm(loader, total=n):
        assert msg.x.to_numpy().shape == (1024, 1024)


bench()



from typing import TypeVar, Iterable, Iterator, Callable, Generic
from io import BufferedIOBase
from tfrecord_slow.reader import TfrecordReader

T = TypeVar("T")


def _default_func(buf: memoryview):
    return buf.tobytes()


class RawTfrecordLoader(Generic[T]):
    def __init__(
        self,
        datapipe: Iterable[BufferedIOBase],
        func: Callable[[memoryview], T] = _default_func,
        check_integrity: bool = False,
    ):
        self.datapipe = datapipe
        self.check_integrity = check_integrity
        self.func = func

    def __iter__(self) -> Iterator[T]:
        for fp in self.datapipe:
            reader = TfrecordReader(fp, check_integrity=self.check_integrity)
            for buf in reader:
                example = self.func(buf)
                yield example



import numpy as np
import msgspec
from tfrecord_slow.utils.msgpack import NdArray, NdArrayView
from tfrecord_slow.utils.numpy import buffer_to_ndarray


def test_encode_decode():
    x = np.arange(100)
    arr = NdArrayView.from_numpy(x)
    buf = msgspec.msgpack.encode(arr)
    assert len(buf) > 0

    x1 = msgspec.msgpack.decode(buf, type=NdArray).to_numpy()
    assert x.dtype == x1.dtype
    assert np.allclose(x, x1)


def test_buffer_to_ndarray():
    shape = (1, 2, 3, 4)
    dtype = np.float32
    x = np.random.rand(*shape).astype(dtype)
    packed = msgspec.msgpack.encode(x.tobytes())
    unpacked = msgspec.msgpack.decode(packed, type=memoryview)
    x1 = buffer_to_ndarray(unpacked, shape=shape, dtype=dtype)
    assert np.allclose(x, x1)

    x2 = buffer_to_ndarray(unpacked)
    assert x2.dtype == np.uint8
    assert x2.ndim == 1




# automatically generated by the FlatBuffers compiler, do not modify

# namespace: Data

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

class NdArray(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = NdArray()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsNdArray(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # NdArray
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # NdArray
    def Data(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 1))
        return 0

    # NdArray
    def DataAsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Uint8Flags, o)
        return 0

    # NdArray
    def DataLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # NdArray
    def DataIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        return o == 0

    # NdArray
    def Shape(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Int64Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 8))
        return 0

    # NdArray
    def ShapeAsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Int64Flags, o)
        return 0

    # NdArray
    def ShapeLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # NdArray
    def ShapeIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        return o == 0

    # NdArray
    def Dtype(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

def Start(builder): builder.StartObject(3)
def NdArrayStart(builder):
    """This method is deprecated. Please switch to Start."""
    return Start(builder)
def AddData(builder, data): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(data), 0)
def NdArrayAddData(builder, data):
    """This method is deprecated. Please switch to AddData."""
    return AddData(builder, data)
def StartDataVector(builder, numElems): return builder.StartVector(1, numElems, 1)
def NdArrayStartDataVector(builder, numElems):
    """This method is deprecated. Please switch to Start."""
    return StartDataVector(builder, numElems)
def AddShape(builder, shape): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(shape), 0)
def NdArrayAddShape(builder, shape):
    """This method is deprecated. Please switch to AddShape."""
    return AddShape(builder, shape)
def StartShapeVector(builder, numElems): return builder.StartVector(8, numElems, 8)
def NdArrayStartShapeVector(builder, numElems):
    """This method is deprecated. Please switch to Start."""
    return StartShapeVector(builder, numElems)
def AddDtype(builder, dtype): builder.PrependUOffsetTRelativeSlot(2, flatbuffers.number_types.UOffsetTFlags.py_type(dtype), 0)
def NdArrayAddDtype(builder, dtype):
    """This method is deprecated. Please switch to AddDtype."""
    return AddDtype(builder, dtype)
def End(builder): return builder.EndObject()
def NdArrayEnd(builder):
    """This method is deprecated. Please switch to End."""
    return End(builder)


from tqdm import tqdm
from common import Message
from tfrecord_slow.loaders.raw_loader import RawTfrecordLoader
from flatbuffers import flexbuffers
import numpy as np


def parse(buf):
    # msg = flexbuffers.GetRoot(bytearray(buf))
    msg = flexbuffers.GetRoot(buf)
    # return {"data": msg.X().Data()}
    # x = np.frombuffer(msg.X().DataAsNumpy().data, dtype=msg.X().Dtype().decode())
    # x = x.reshape(msg.X().ShapeAsNumpy())

    # print(msg.IsMap)
    # map = msg.AsMap
    # xmap = map["x"].AsMap

    # x = np.frombuffer(xmap["data"].AsBlob, dtype=xmap["dtype"].AsString)
    # x = x.reshape(xmap["shape"].AsVector.Value)

    # data = msg.AsVector[0].AsBlob
    vec = flexbuffers.Vector(msg._Indirect(), msg._byte_width)
    data = flexbuffers.Blob(vec[0]._Indirect(), vec[0]._byte_width).Bytes

    # print(type(data))
    x = np.frombuffer(data, dtype=np.float64).reshape(1024, 1024)

    return x


def bench(n: int = 1000):
    # with TfRecordReader.open("/tmp/test_writer.tfrec") as reader:
    loader = RawTfrecordLoader([open("/tmp/test_flex_writer.tfrec", "rb")], parse)
    for msg in tqdm(loader, total=n):
        # assert msg.x.to_numpy().shape == (1024, 1024)
        # assert msg.shape == (1024, 1024)
        pass


bench()



from tfrecord_slow import TfrecordWriter
from tqdm import tqdm
import numpy as np

import flatbuffers
import Data.Message
import Data.NdArray


def create_message(builder: flatbuffers.Builder, x: np.ndarray):
    builder.Clear()

    fbs_dtype = builder.CreateString(x.dtype.str)
    fbs_shape = builder.CreateNumpyVector(np.array(x.shape, dtype=np.int64))
    fbs_data = builder.CreateByteVector(x.data.tobytes())

    Data.NdArray.Start(builder)
    Data.NdArray.AddData(builder, fbs_data)
    Data.NdArray.AddShape(builder, fbs_shape)
    Data.NdArray.AddDtype(builder, fbs_dtype)
    fbs_x = Data.NdArray.End(builder)

    Data.Message.Start(builder)
    Data.Message.AddX(builder, fbs_x)
    fbs_msg = Data.Message.End(builder)

    builder.Finish(fbs_msg)
    return builder.Output()


def bench(n: int = 1000):
    x = np.random.rand(1024, 1024)
    builder = flatbuffers.Builder()
    with TfrecordWriter.create("/tmp/test_fbs_writer.tfrec") as writer:
        for _ in tqdm(range(n)):
            writer.write(create_message(builder, x))


bench()



"""
Make a new package for it.
"""

import argparse
from pathlib import Path
from typing import Optional
from .reader import TfrecordReader
from loguru import logger
import time


def get_args():
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers()
    add_count_args(subparsers.add_parser("count"))

    return parser.parse_args()


def add_count_args(parser: argparse.ArgumentParser):
    parser.set_defaults(func=count)
    parser.add_argument("path", type=Path, help="Path to tfrecord file or directory.")
    parser.add_argument("-m", "--mask", action="store_const", const="*.tfrec")
    parser.add_argument("-c", "--check", action="store_true", help="Check integrity.")


def count(path: Path, mask: Optional[str], check: bool, **kwargs):
    if mask is not None:
        paths = list(path.glob(mask))
    else:
        paths = [path]

    total = 0

    for path in paths:
        with TfrecordReader.open(str(path), check_integrity=check) as reader:
            start_time = time.perf_counter()
            num_records = reader.count()
            end_time = time.perf_counter()
            records_per_second = num_records / (end_time - start_time)
            logger.info(
                f"file: {path}, records: {num_records}, speed: {records_per_second} rec/s"
            )
            total += num_records

    logger.info("total records: {}", total)


def main():
    args = get_args()
    logger.debug("{}", args)
    if "func" in args:
        args.func(**vars(args))


if __name__ == "__main__":
    main()
